nextflow.enable.dsl = 2

/*
 * Channels
*/
Channel
  .fromPath( params.fastq1 )
  .set { fastq1 }
Channel
  .fromPath( params.fastq2 )
  .set { fastq2 }
Channel
  .fromPath( params.reference_fasta )
  .set{ reference_fasta }

/*
 * Includes
 */
include {
  FASTQC as FASTQC_RAW_DATA
} from "./modules/fastqc"
include {
  FASTP
} from "./modules/fastp"
include {
  FASTQC as FASTQC_PRE_PROCESSED
} from "./modules/fastqc"
include {
  BWA_INDEX
} from "./modules/bwa_index"
include {
  SAMTOOLS_FAIDX
} from "./modules/samtools_faidx"

/*
* Subworkflow bwa_index
*/
workflow SUB_BWA_INDEX {
  main:
    BWA_INDEX(reference_fasta)
    SAMTOOLS_FAIDX(reference_fasta, BWA_INDEX.out)
  emit:
    output_BWA_INDEX = SAMTOOLS_FAIDX.out
}

/*
* Subworkflow trimmed_fastq
* Quality Control (raw data), Raw Data trimming and Quality Control (pre-processed)
*/
workflow TRIMMED_FASTQ {
  main:
    FASTQC_RAW_DATA(fastq1,
                    fastq2,
                    params.threads_fastqc)

    FASTP(fastq1,
          fastq2,
          params.threads_fastp,
          params.cut_right_window_size,
          params.cut_right_mean_quality,
          params.trim_tail1,
          params.length_required)

    FASTQC_PRE_PROCESSED(FASTP.out.paired_fastq1,
                         FASTP.out.paired_fastq2,
                         params.threads_fastqc)
}

workflow{
  TRIMMED_FASTQ()
  SUB_BWA_INDEX()
}
